@@ -1222,7 +1222,7 @@
  (constructor_declaration_se
   (lambda_match
    ((Sexpr loc [(Suid _ ci) . sel])
-    (values loc <:vala< (rename_id ci) >> <:vala< (List.map ctyp_se sel) >>
+    (values loc <:vala< (rename_id ci) >> <:vala< [] >> <:vala< (List.map ctyp_se sel) >>
        <:vala< None >> <:vala< [] >>))
    (se (error se "constructor_declaration"))))
  (variant_declaration_se
