@@ -878,30 +878,44 @@ value label_decl pc (loc, l, m, t, attrs) =
     (hlist (pr_attribute "@")) (Pcaml.unvala attrs)
 ;
 
+value typevars_binder pc = fun [
+  [] -> pprintf pc ""
+| l -> pprintf pc "%p . " (hlist typevar) l
+]
+;
+
 value cons_decl pc = fun [
-  <:constructor< $_uid:c$ of $_list:tl$ $_rto:rto$ $_algattrs:alg_attrs$ >> ->
+  <:constructor< $_uid:c$ of $_list:tyvars$ . $_list:tl$ $_rto:rto$ $_algattrs:alg_attrs$ >>
+ ->
   let c = Pcaml.unvala c in
   let tl = Pcaml.unvala tl in
   if tl = [] then do {
-    match Pcaml.unvala rto with
-    [ Some rt -> pprintf pc "%p : %p%p" cons_escaped c ctyp_below_alg_attribute rt
-                   (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
-    | None -> pprintf pc "%p%p" cons_escaped c
-                   (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
+    match (Pcaml.unvala tyvars, Pcaml.unvala rto) with
+      [ ([], Some rt) -> pprintf pc "%p : %p%p" cons_escaped c ctyp_below_alg_attribute rt
+                     (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
+      | (l, Some rt) -> pprintf pc "%p : %p%p%p" cons_escaped c typevars_binder l ctyp_below_alg_attribute rt
+                     (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
+      | (_, None) -> pprintf pc "%p%p" cons_escaped c
+                  (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
     ]
   }
   else do {
-    match Pcaml.unvala rto with
-    [ Some rt ->
+    match (Pcaml.unvala tyvars, Pcaml.unvala rto) with
+      [ ([], Some rt) ->
         let tl = List.map (fun t -> (t, " and")) tl in
         pprintf pc "%p of@;<1 4>%p : %p%p" cons_escaped c (plist ctyp_below_alg_attribute 2) tl
           ctyp_below_alg_attribute rt
           (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
-    | None ->
+      | (l, Some rt) ->
         let tl = List.map (fun t -> (t, " and")) tl in
-        pprintf pc "%p of@;<1 4>%p%p" cons_escaped c (plist ctyp_below_alg_attribute 2) tl
+        pprintf pc "%p of@;<1 4>%p%p : %p%p" cons_escaped c typevars_binder l (plist ctyp_below_alg_attribute 2) tl
+          ctyp_below_alg_attribute rt
           (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
-    ]
+      | (_, None) ->
+         let tl = List.map (fun t -> (t, " and")) tl in
+         pprintf pc "%p of@;<1 4>%p%p" cons_escaped c (plist ctyp_below_alg_attribute 2) tl
+           (hlist (pr_attribute "@")) (Pcaml.unvala alg_attrs)
+      ]
   }
 ]
 ;
@@ -920,7 +934,7 @@ value extension_constructor loc pc ec = match ec with [
 value has_ecs_with_params vdl =
   List.exists
     (fun [
-       MLast.EcTuple _ (_, _, tl, rto,_) ->
+       MLast.EcTuple _ (_, _, _, tl, rto,_) ->
        match tl with
          [ <:vala< [] >> -> False
          | _ -> True ]
@@ -970,7 +984,7 @@ value type_extension loc pc te =
 
 value has_cons_with_params vdl =
   List.exists
-    (fun (_, _, tl, _,_) ->
+    (fun (_, _, _, tl, _,_) ->
        match tl with
        [ <:vala< [] >> -> False
        | _ -> True ])
@@ -1146,14 +1160,14 @@ value expr_short pc x =
 value string pc s = pprintf pc "\"%s\"" s;
 value lident pc s = pprintf pc "%s" s;
 
-value external_decl pc (loc, n, t, sl, attrs) =
-  pprintf pc "external %p :@;%p = %s%p" var_escaped (loc, n) ctyp t
+value external_decl pc (loc, n, tyvars, t, sl, attrs) =
+  pprintf pc "external %p :@;%p%p = %s%p" var_escaped (loc, n) typevars_binder tyvars ctyp t
     (hlist string {(pc) with bef = ""; aft = ""} sl)
     (hlist (pr_attribute "@@")) attrs
 ;
 
-value external_decl_original pc (loc, n, t, sl, attrs) =
-  pprintf pc "external ( %s ) :@;%p = %s%p" n ctyp t
+value external_decl_original pc (loc, n, tyvars, t, sl, attrs) =
+  pprintf pc "external ( %s ) :@;%p%p = %s%p" n typevars_binder tyvars ctyp t
     (hlist string {(pc) with bef = ""; aft = ""} sl)
     (hlist (pr_attribute "@@")) attrs
 ;
@@ -2014,11 +2028,12 @@ EXTEND_PRINTER
       | <:str_item:< exception $excon:ec$ $_itemattrs:item_attrs$ >> ->
           pprintf pc "exception %p%p" (extension_constructor loc) ec
             (hlist (pr_attribute "@@")) (Pcaml.unvala item_attrs)
-      | <:str_item:< external $lid:n$ : $t$ = $list:sl$ $itemattrs:attrs$ >> ->
+
+      | <:str_item:< external $lid:n$ : $list:tyvars$ . $t$ = $list:sl$ $itemattrs:attrs$ >> ->
           if is_special_op n then
-            external_decl_original pc (loc, n, t, sl, attrs)
+            external_decl_original pc (loc, n, tyvars, t, sl, attrs)
           else
-            external_decl pc (loc, n, t, sl, attrs)
+            external_decl pc (loc, n, tyvars, t, sl, attrs)
       | <:str_item< include $me$ $_itemattrs:attrs$ >> ->
           pprintf pc "include %p%p" module_expr me (hlist (pr_attribute "@@")) (Pcaml.unvala attrs)
       | <:str_item< module $flag:rf$ $list:mdl$ >> ->
@@ -2076,11 +2091,12 @@ EXTEND_PRINTER
                    (vlist (semi_after sig_item)) sil)
       | MLast.SgExc _ gc item_attrs -> pprintf pc "exception %p%p" cons_decl gc
             (hlist (pr_attribute "@@")) (Pcaml.unvala item_attrs)
-      | <:sig_item:< external $lid:n$ : $t$ = $list:sl$ $itemattrs:attrs$ >> ->
+
+      | <:sig_item:< external $lid:n$ : $list:tyvars$ . $t$ = $list:sl$ $itemattrs:attrs$ >> ->
           if is_special_op n then
-            external_decl_original pc (loc, n, t, sl, attrs)
+            external_decl_original pc (loc, n, tyvars, t, sl, attrs)
           else
-            external_decl pc (loc, n, t, sl, attrs)
+            external_decl pc (loc, n, tyvars, t, sl, attrs)
       | <:sig_item< include $mt$ $_itemattrs:item_attrs$ >> ->
           pprintf pc "include %p%p" module_type mt (hlist (pr_attribute "@@")) (Pcaml.unvala item_attrs)
       | <:sig_item< module $flag:rf$ $list:mdl$ >> ->
@@ -2102,10 +2118,19 @@ EXTEND_PRINTER
           pprintf pc "type%s %p" (if nonrf then " nonrec" else "") (vlist2 type_decl (and_before type_decl)) tdl
       | MLast.SgTypExten loc te ->
           pprintf pc "type %p" (type_extension loc) te
+
+      | <:sig_item:< value $lid:s$ : ! $list:ls$ . $t$ $itemattrs:attrs$ >> when is_special_op s ->
+          pprintf pc "value ( %s ) :@;%p%p%p" s typevars_binder ls ctyp t (hlist (pr_attribute "@@")) attrs
+
       | <:sig_item:< value $lid:s$ : $t$ $itemattrs:attrs$ >> when is_special_op s ->
           pprintf pc "value ( %s ) :@;%p%p" s ctyp t (hlist (pr_attribute "@@")) attrs
+
+      | <:sig_item:< value $lid:s$ : ! $list:ls$ . $t$ $itemattrs:attrs$ >> ->
+          pprintf pc "value %p :@;%p%p%p" var_escaped (loc, s) typevars_binder ls ctyp t (hlist (pr_attribute "@@")) attrs
+
       | <:sig_item:< value $lid:s$ : $t$ $itemattrs:attrs$ >> ->
           pprintf pc "value %p :@;%p%p" var_escaped (loc, s) ctyp t (hlist (pr_attribute "@@")) attrs
+
       | <:sig_item< class type $list:_$ >> | <:sig_item< class $list:_$ >> ->
           failwith "classes and objects not pretty printed; add pr_ro.cmo"
       | MLast.SgUse _ fn sl ->
